@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Roger Light <roger@atchoo.org>
+Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -101,6 +101,7 @@ POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
+#include <errno.h>
 #include <sqlite3.h>
 #include <stdio.h>
 #include <string.h>
@@ -110,6 +111,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <mqtt3.h>
 
 static sqlite3 *db = NULL;
+static char *db_filepath = NULL;
 
 /* Need to make a struct here to have an array.
  * Don't know size of sqlite3_stmt so can't array it directly.
@@ -134,74 +136,94 @@ int _mqtt3_db_transaction_begin(void);
 int _mqtt3_db_transaction_end(void);
 int _mqtt3_db_transaction_rollback(void);
 
-int mqtt3_db_open(const char *location, const char *filename, const char *regex_ext_path)
-{
-	char *filepath;
-	char *errmsg;
+#ifdef WITH_CLIENT
+/* Client callback for publish events - this WILL change. */
+int (*client_publish_callback)(const char *, int, uint32_t, const uint8_t *, int) = NULL;
+#endif
 
-	if(!filename) return 1;
+int mqtt3_db_open(mqtt3_config *config)
+{
+	char *errmsg;
+	int dbrc;
+	int rc = 0;
+	sqlite3_backup *restore;
+	sqlite3 *restore_db;
+	FILE *fptr;
+
+	if(!config) return 1;
 #ifdef WITH_REGEX
-	if(!regex_ext_path) return 1;
+	if(!config->ext_sqlite_regex) return 1;
 #endif
 	if(sqlite3_initialize() != SQLITE_OK){
 		return 1;
 	}
 
-	if(!strcmp(filename, ":memory:")){
-		if(sqlite3_open_v2(filename, &db, SQLITE_OPEN_READWRITE, NULL) != SQLITE_OK){
-			mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", sqlite3_errmsg(db));
-			return 1;
-		}
+	if(sqlite3_open_v2(":memory:", &db, SQLITE_OPEN_READWRITE, NULL) != SQLITE_OK){
+		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", sqlite3_errmsg(db));
+		return 1;
+	}
+
+	if(!config->persistence){
 		if(_mqtt3_db_tables_create()) return 1;
 	}else{
-		if(location && strlen(location)){
-			filepath = mqtt3_malloc(strlen(location) + strlen(filename) + 1);
-			if(!filepath) return 1;
-			sprintf(filepath, "%s%s", location, filename);
+		if(config->persistence_location && strlen(config->persistence_location)){
+			db_filepath = mqtt3_malloc(strlen(config->persistence_location) + strlen("mosquitto.db") + 1);
+			if(!db_filepath) return 1;
+			sprintf(db_filepath, "%smosquitto.db", config->persistence_location);
 		}else{
-			filepath = (char *)filename;
+			db_filepath = mqtt3_strdup("mosquitto.db");
 		}
-		/* Open without creating first. If found, check for db version.
-		 * If not found, open with create.
-		 */
-		if(sqlite3_open_v2(filepath, &db, SQLITE_OPEN_READWRITE, NULL) == SQLITE_OK){
-			if(_mqtt3_db_version_check()){
-				mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid database version.");
-				return 1;
+		dbrc = sqlite3_open_v2(db_filepath, &restore_db, SQLITE_OPEN_READONLY, NULL);
+		/* FIXME - need to handle all error conditions, *especially* file doesn't exist. */
+		if(dbrc == SQLITE_OK){
+			restore = sqlite3_backup_init(db, "main", restore_db, "main");
+			if(restore){
+				sqlite3_backup_step(restore, -1);
+				sqlite3_backup_finish(restore);
+				if(_mqtt3_db_version_check()){
+					mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid database version.");
+					rc = 1;
+				}
+			}else{
+				mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Couldn't restore database %s (%d).", db_filepath, dbrc);
+				rc = 1;
 			}
+			sqlite3_close(restore_db);
 		}else{
-			if(sqlite3_open_v2(filepath, &db,
-					SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL) != SQLITE_OK){
-				mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", sqlite3_errmsg(db));
-				return 1;
+			fptr = fopen(db_filepath, "rb");
+			if(!fptr){
+				switch(errno){
+					case ENOENT:
+						/* File doesn't exist - ok to create */
+						if(_mqtt3_db_tables_create()){
+							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Unable to populate new database. Try restarting mosquitto.");
+							return 1;
+						}
+						break;
+					case EACCES:
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Permission denied trying to restore persistent database %s.", db_filepath);
+						return 1;
+				}
+			}else{
+				mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Possibly corrupt database file. Try restarting mosquitto.");
+				fclose(fptr);
 			}
-			if(_mqtt3_db_tables_create()) return 1;
-		}
-		if(filepath && filepath != filename){
-			mqtt3_free(filepath);
 		}
 	}
 
 #ifdef WITH_REGEX
 	sqlite3_enable_load_extension(db, 1);
-	if(sqlite3_load_extension(db, regex_ext_path, NULL, &errmsg) != SQLITE_OK){
+	if(sqlite3_load_extension(db, config->ext_sqlite_regex, NULL, &errmsg) != SQLITE_OK){
 		if(errmsg){
 			mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", errmsg);
 			sqlite3_free(errmsg);
 		}
-		mqtt3_log_printf(MQTT3_LOG_NOTICE, "Retrying with regex_ext_path=/usr/lib/sqlite3-pcre.so");
-		if(sqlite3_load_extension(db, "/usr/lib/sqlite3-pcre.so", NULL, &errmsg) != SQLITE_OK){
-			if(errmsg){
-				mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", errmsg);
-				sqlite3_free(errmsg);
-			}
-			return 1;
-		}
+		return 1;
 	}
 #endif
 	if(_mqtt3_db_invalidate_sockets()) return 1;
 
-	return 0;
+	return rc;
 }
 
 int mqtt3_db_close(void)
@@ -221,6 +243,30 @@ int mqtt3_db_close(void)
 	return 0;
 }
 
+int mqtt3_db_backup(void)
+{
+	int rc = 0;
+	sqlite3 *backup_db;
+	sqlite3_backup *backup;
+
+	if(!db || !db_filepath) return 1;
+	mqtt3_log_printf(MQTT3_LOG_INFO, "Saving in-memory database to %s.", db_filepath);
+	if(sqlite3_open(db_filepath, &backup_db) != SQLITE_OK){
+		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Unable to open on-disk database for writing.");
+		return 1;
+	}
+	backup = sqlite3_backup_init(backup_db, "main", db, "main");
+	if(backup){
+		sqlite3_backup_step(backup, -1);
+		sqlite3_backup_finish(backup);
+	}else{
+		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Unable to save in-memory database to disk.");
+		rc = 1;
+	}
+	sqlite3_close(backup_db);
+	return rc;
+}
+
 int _mqtt3_db_tables_create(void)
 {
 	int rc = 0;
@@ -507,6 +553,34 @@ int mqtt3_db_client_will_queue(mqtt3_context *context)
 	return rc;
 }
 
+/* Returns the number of client currently in the database.
+ * This includes inactive clients.
+ * Returns 1 on failure (count is NULL, sqlite error)
+ * Returns 0 on success.
+ */
+int mqtt3_db_client_count(int *count)
+{
+	int rc = 0;
+	static sqlite3_stmt *stmt = NULL;
+
+	if(!count) return 1;
+
+	if(!stmt){
+		stmt = _mqtt3_db_statement_prepare("SELECT COUNT(*) FROM clients");
+		if(!stmt){
+			return 1;
+		}
+	}
+	if(sqlite3_step(stmt) == SQLITE_ROW){
+		*count = sqlite3_column_int(stmt, 0);
+	}else{
+		rc = 1;
+	}
+	sqlite3_reset(stmt);
+
+	return rc;
+}
+
 /* Delete a client from the database.
  * Called when clients with clean start enabled disconnect.
  * Returns 1 on failure (context or context->id is NULL, sqlite error)
@@ -785,6 +859,11 @@ int mqtt3_db_messages_queue(const char *topic, int qos, uint32_t payloadlen, con
 	/* Find all clients that subscribe to topic and put messages into the db for them. */
 	if(!topic || !payloadlen || !payload) return 1;
 
+#ifdef WITH_CLIENT
+	if(client_publish_callback){
+		client_publish_callback(topic, qos, payloadlen, payload, retain);
+	}
+#endif
 	if(retain){
 		if(mqtt3_db_retain_insert(topic, qos, payloadlen, payload)) rc = 1;
 	}
@@ -814,7 +893,6 @@ int mqtt3_db_messages_queue(const char *topic, int qos, uint32_t payloadlen, con
 			if(client_id) mqtt3_free(client_id);
 		}
 	}
-
 	return rc;
 }
 
@@ -1444,9 +1522,13 @@ int mqtt3_db_sub_search_start(const char *topic)
 
 	if(!stmt_sub_search){
 #ifdef WITH_REGEX
-		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs WHERE regexp(?, sub)");
+		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs "
+				"JOIN clients ON subs.client_id=clients.id "
+				"WHERE (subs.qos<>0 OR clients.sock<>-1) AND regexp(?, subs.sub)");
 #else
-		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs where sub=?");
+		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs "
+				"JOIN clients ON subs.client_id=clients.id "
+				"WHERE subs.sub=? AND (subs.qos<>0 OR clients.sock<>-1)");
 #endif
 		if(!stmt_sub_search){
 			return 1;
@@ -1533,6 +1615,11 @@ void mqtt3_db_sys_update(int interval, time_t start_time)
 			mqtt3_db_messages_queue("$SYS/broker/messages/inflight", 2, strlen(buf), (uint8_t *)buf, 1);
 		}
 
+		if(!mqtt3_db_client_count(&count)){
+			snprintf(buf, 100, "%d", count);
+			mqtt3_db_messages_queue("$SYS/broker/clients/total", 2, strlen(buf), (uint8_t *)buf, 1);
+		}
+
 		snprintf(buf, 100, "%d", mqtt3_memory_used());
 		mqtt3_db_messages_queue("$SYS/broker/heap/current size", 2, strlen(buf), (uint8_t *)buf, 1);
 
