@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Roger Light <roger@atchoo.org>
+Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,12 +30,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef MQTT3_H
 #define MQTT3_H
 
+#include <config.h>
+
 #include <stdbool.h>
 #include <stdint.h>
 #include <syslog.h>
 #include <sys/select.h>
 #include <time.h>
 
+#ifndef __GNUC__
+#define __attribute__(attrib)
+#endif
+
 /* For version 3 of the MQTT protocol */
 
 #define PROTOCOL_NAME "MQIsdp"
@@ -82,6 +88,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 struct _mqtt3_packet{
 	uint8_t command;
+#ifdef WITH_CLIENT
+	uint8_t command_saved;
+#endif
 	uint8_t have_remaining;
 	uint8_t remaining_count;
 	uint32_t remaining_mult;
@@ -98,6 +107,7 @@ typedef struct _mqtt3_context{
 	time_t last_msg_out;
 	uint16_t keepalive;
 	bool clean_start;
+	bool connected;
 	char *id;
 	char *address;
 	struct _mqtt3_packet in_packet;
@@ -128,20 +138,30 @@ struct mqtt3_iface {
 };
 
 typedef struct {
-	int daemon;
+	int autosave_interval;
+	bool daemon;
 	char *ext_sqlite_regex;
 	struct mqtt3_iface *iface;
 	int iface_count;
 	int log_dest;
 	int log_type;
-	int msg_timeout;
-	int persistence;
+	bool persistence;
 	char *persistence_location;
+	int retry_interval;
 	int sys_interval;
 	char *pid_file;
 	char *user;
 } mqtt3_config;
 
+#ifdef WITH_CLIENT
+/* Client callback for publish events - this WILL change. */
+extern int (*client_publish_callback)(const char *, int, uint32_t, const uint8_t *, int);
+/* Client callback for connack events - this WILL change. */
+extern void (*client_connack_callback)(int);
+/* Client callback for when a packet has been successfully written. */
+extern void (*client_net_write_callback)(int);
+#endif
+
 /* ============================================================
  * Utility functions
  * ============================================================ */
@@ -186,7 +206,7 @@ int mqtt3_send_simple_command(mqtt3_context *context, uint8_t command);
  * Network functions
  * ============================================================ */
 int mqtt3_socket_accept(mqtt3_context **contexts, int context_count, int listensock);
-int mqtt3_socket_connect(const char *ip, uint16_t port);
+int mqtt3_socket_connect(const char *host, uint16_t port);
 int mqtt3_socket_close(mqtt3_context *context);
 int mqtt3_socket_listen(uint16_t port);
 int mqtt3_socket_listen_if(const char *iface, uint16_t port);
@@ -231,10 +251,12 @@ int mqtt3_handle_unsubscribe(mqtt3_context *context);
 /* ============================================================
  * Database handling
  * ============================================================ */
-int mqtt3_db_open(const char *location, const char *filename, const char *regex_ext_path);
+int mqtt3_db_open(mqtt3_config *config);
 int mqtt3_db_close(void);
+int mqtt3_db_backup(void);
 int mqtt3_db_client_insert(mqtt3_context *context, int will, int will_retain, int will_qos, const char *will_topic, const char *will_message);
 int mqtt3_db_client_update(mqtt3_context *context, int will, int will_retain, int will_qos, const char *will_topic, const char *will_message);
+int mqtt3_db_client_count(int *count);
 /* Remove the client detailed in context from the clients table only. */
 int mqtt3_db_client_delete(mqtt3_context *context);
 /* Return the socket number in sock of a client client_id. */
@@ -294,6 +316,6 @@ char *mqtt3_strdup(const char *s);
  * ============================================================ */
 int mqtt3_log_init(int level, int destinations);
 int mqtt3_log_close(void);
-int mqtt3_log_printf(int level, const char *fmt, ...);
+int mqtt3_log_printf(int level, const char *fmt, ...) __attribute__((format(printf, 2, 3)));
 
 #endif
