@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Roger Light <roger@atchoo.org>
+Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -45,46 +45,42 @@ int mqtt3_packet_handle(mqtt3_context *context)
 	if(!context) return 1;
 
 	switch((context->in_packet.command)&0xF0){
-		case CONNECT:
-			if(mqtt3_handle_connect(context)) return 1;
-			break;
-		case DISCONNECT:
-			if(mqtt3_handle_disconnect(context)) return 1;
-			break;
 		case PINGREQ:
-			if(mqtt3_handle_pingreq(context)) return 1;
-			break;
+			return mqtt3_handle_pingreq(context);
 		case PINGRESP:
-			if(mqtt3_handle_pingresp(context)) return 1;
-			break;
+			return mqtt3_handle_pingresp(context);
 		case PUBACK:
-			if(mqtt3_handle_puback(context)) return 1;
-			break;
+			return mqtt3_handle_puback(context);
 		case PUBCOMP:
-			if(mqtt3_handle_pubcomp(context)) return 1;
-			break;
+			return mqtt3_handle_pubcomp(context);
 		case PUBLISH:
-			if(mqtt3_handle_publish(context)) return 1;
-			break;
+			return mqtt3_handle_publish(context);
 		case PUBREC:
-			if(mqtt3_handle_pubrec(context)) return 1;
-			break;
+			return mqtt3_handle_pubrec(context);
 		case PUBREL:
-			if(mqtt3_handle_pubrel(context)) return 1;
-			break;
+			return mqtt3_handle_pubrel(context);
+#ifdef WITH_BROKER
+		case CONNECT:
+			return mqtt3_handle_connect(context);
+		case DISCONNECT:
+			return mqtt3_handle_disconnect(context);
 		case SUBSCRIBE:
-			if(mqtt3_handle_subscribe(context)) return 1;
-			break;
+			return mqtt3_handle_subscribe(context);
 		case UNSUBSCRIBE:
-			if(mqtt3_handle_unsubscribe(context)) return 1;
-			break;
+			return mqtt3_handle_unsubscribe(context);
+#endif
+#ifdef WITH_CLIENT
+		case CONNACK:
+			return mqtt3_handle_connack(context);
+		case SUBACK:
+			return mqtt3_handle_suback(context);
+		case UNSUBACK:
+			return mqtt3_handle_unsuback(context);
+#endif
 		default:
 			/* If we don't recognise the command, return an error straight away. */
 			return 1;
-			break;
 	}
-
-	return 0;
 }
 
 int mqtt3_handle_puback(mqtt3_context *context)
